@@ -7,13 +7,14 @@ from peek_plugin_graphdb._private.PluginNames import graphDbTuplePrefix
 
 @addTupleType
 class GraphDbPackedItemKeyTuple(Tuple):
-    """ Packed Item Key
+    """Packed Item Key
 
     This tuple transports the Segment JSON that is packed in the encoded ItemKey tuple.
 
     """
-    __tupleType__ = graphDbTuplePrefix + 'GraphDbPackedItemKeyTuple'
-    __rawJonableFields__ = ['key', 'jsonDict']
+
+    __tupleType__ = graphDbTuplePrefix + "GraphDbPackedItemKeyTuple"
+    __rawJonableFields__ = ["key", "jsonDict"]
 
     #:  The unique key of this segment
     key: str = None
