@@ -9,6 +9,7 @@ import {
     TupleOfflineStorageNameService,
     TupleOfflineStorageService,
     TupleSelector,
+    TupleStorageBatchSaveArguments,
     TupleStorageFactoryService,
     VortexService,
     VortexStatusService,
@@ -27,7 +28,7 @@ import { GraphDbModelSetTuple } from "../../GraphDbModelSetTuple";
 import { GraphDbTupleService } from "../GraphDbTupleService";
 import { GraphDbPackedItemKeyTuple } from "./GraphDbPackedItemKeyTuple";
 import {
-    DeviceOfflineCacheControllerService,
+    DeviceOfflineCacheService,
     OfflineCacheStatusTuple,
 } from "@peek/peek_core_device";
 
@@ -114,7 +115,12 @@ function keyChunk(modelSetKey: string, key: string): string {
 @Injectable()
 export class ItemKeyIndexLoaderService extends NgLifeCycleEvents {
     private UPDATE_CHUNK_FETCH_SIZE = 32;
-    private OFFLINE_CHECK_PERIOD_MS = 15 * 60 * 1000; // 15 minutes
+
+    // Every 100 chunks from the server
+    private SAVE_POINT_ITERATIONS = 100;
+
+    // Saving the cache after each chunk is so expensive, we only do it every 20 or so
+    private chunksSavedSinceLastIndexSave = 0;
 
     private index = new ItemKeyIndexUpdateDateTuple();
     private askServerChunks: ItemKeyIndexUpdateDateTuple[] = [];
@@ -135,13 +141,15 @@ export class ItemKeyIndexLoaderService extends NgLifeCycleEvents {
         private vortexStatusService: VortexStatusService,
         storageFactory: TupleStorageFactoryService,
         private tupleService: GraphDbTupleService,
-        private deviceCacheControllerService: DeviceOfflineCacheControllerService
+        private deviceCacheControllerService: DeviceOfflineCacheService
     ) {
         super();
 
         this.storage = new TupleOfflineStorageService(
             storageFactory,
-            new TupleOfflineStorageNameService(graphDbCacheStorageName)
+            new TupleOfflineStorageNameService(
+                graphDbCacheStorageName + ".item_key"
+            )
         );
 
         this.setupVortexSubscriptions();
@@ -186,11 +194,11 @@ export class ItemKeyIndexLoaderService extends NgLifeCycleEvents {
             (tuples: ItemKeyTuple[]) => {
                 let keys = [];
                 for (let tuple of tuples) {
-                    if (tuple.itemType == itemType) {
-                        for (let segmentKey of tuple.segmentKeys) {
-                            keys.push(segmentKey);
-                        }
+                    // if (tuple.itemType == itemType) {
+                    for (let segmentKey of tuple.segmentKeys) {
+                        keys.push(segmentKey);
                     }
+                    // }
                 }
                 return keys;
             }
@@ -261,7 +269,7 @@ export class ItemKeyIndexLoaderService extends NgLifeCycleEvents {
             )
             .pipe(takeUntil(this.onDestroyEvent))
             .subscribe((payloadEnvelope: PayloadEnvelope) => {
-                this.processItemKeyIndexsFromServer(payloadEnvelope);
+                this.processChunksFromServer(payloadEnvelope);
             });
 
         // If the vortex service comes back online, update the watch grids.
@@ -273,7 +281,7 @@ export class ItemKeyIndexLoaderService extends NgLifeCycleEvents {
 
     private areWeTalkingToTheServer(): boolean {
         return (
-            this.deviceCacheControllerService.cachingEnabled &&
+            this.deviceCacheControllerService.offlineModeEnabled &&
             this.vortexStatusService.snapshot.isOnline
         );
     }
@@ -357,95 +365,103 @@ export class ItemKeyIndexLoaderService extends NgLifeCycleEvents {
 
         if (this.askServerChunks.length == 0) return;
 
-        let indexChunk: ItemKeyIndexUpdateDateTuple =
-            this.askServerChunks.pop();
-        let filt = extend({}, clientItemKeyIndexWatchUpdateFromDeviceFilt);
-        filt[cacheAll] = true;
-        let pl = new Payload(filt, [indexChunk]);
-        this.vortexService.sendPayload(pl);
+        this.deviceCacheControllerService //
+            .waitForGarbageCollector()
+            .then(() => {
+                let indexChunk: ItemKeyIndexUpdateDateTuple =
+                    this.askServerChunks.pop();
+                let filt = extend(
+                    {},
+                    clientItemKeyIndexWatchUpdateFromDeviceFilt
+                );
+                filt[cacheAll] = true;
+                let pl = new Payload(filt, [indexChunk]);
+                this.vortexService.sendPayload(pl);
 
-        this._status.lastCheck = new Date();
-        this._notifyStatus();
+                this._status.lastCheck = new Date();
+                this._notifyStatus();
+            });
     }
 
     /** Process ItemKeyIndexes From Server
      *
      * Process the grids the server has sent us.
      */
-    private processItemKeyIndexsFromServer(payloadEnvelope: PayloadEnvelope) {
+    private async processChunksFromServer(
+        payloadEnvelope: PayloadEnvelope
+    ): Promise<void> {
         if (payloadEnvelope.result != null && payloadEnvelope.result != true) {
             console.log(`ERROR: ${payloadEnvelope.result}`);
             return;
         }
 
-        payloadEnvelope
-            .decodePayload()
-            .then((payload: Payload) => this.storeItemKeyIndexPayload(payload))
-            .then(() => {
-                if (this.askServerChunks.length == 0) {
-                    this.index.initialLoadComplete = true;
-                    this._hasLoaded = true;
-                    this._hasLoadedSubject.next();
-                } else if (payloadEnvelope.filt[cacheAll] == true) {
-                    this.askServerForNextUpdateChunk();
-                }
-            })
-            .then(() => this._notifyStatus())
-            .catch(
-                (e) =>
-                    `ItemKeyIndexCache.processItemKeyIndexsFromServer failed: ${e}`
-            );
-    }
-
-    private storeItemKeyIndexPayload(payload: Payload) {
-        let tuplesToSave: ItemKeyIndexEncodedChunkTuple[] = <
+        const tuplesToSave: ItemKeyIndexEncodedChunkTuple[] = <
             ItemKeyIndexEncodedChunkTuple[]
-        >payload.tuples;
-        if (tuplesToSave.length == 0) return;
+        >payloadEnvelope.data;
 
-        // 2) Store the index
-        this.storeItemKeyIndexChunkTuples(tuplesToSave)
-            .then(() => {
-                // 3) Store the update date
+        try {
+            await this.storeChunkTuples(tuplesToSave);
+        } catch (e) {
+            console.log(`ItemKeyIndexCache.storeChunkTuples: ${e}`);
+        }
 
-                for (let graphDbIndex of tuplesToSave) {
-                    this.index.updateDateByChunkKey[graphDbIndex.chunkKey] =
-                        graphDbIndex.lastUpdate;
-                }
+        if (this.askServerChunks.length == 0) {
+            this.index.initialLoadComplete = true;
+            await this.saveChunkCacheIndex(true);
+            this._hasLoaded = true;
+            this._hasLoadedSubject.next();
+        } else if (payloadEnvelope.filt[cacheAll] == true) {
+            this.askServerForNextUpdateChunk();
+        }
 
-                return this.storage.saveTuples(new UpdateDateTupleSelector(), [
-                    this.index,
-                ]);
-            })
-            .catch((e) =>
-                console.log(`ItemKeyIndexCache.storeItemKeyIndexPayload: ${e}`)
-            );
+        this._notifyStatus();
     }
 
     /** Store Index Bucket
      * Stores the index bucket in the local db.
      */
-    private storeItemKeyIndexChunkTuples(
-        encodedItemKeyIndexChunkTuples: ItemKeyIndexEncodedChunkTuple[]
+    private async storeChunkTuples(
+        tuplesToSave: ItemKeyIndexEncodedChunkTuple[]
     ): Promise<void> {
-        let retPromise: any;
-        retPromise = this.storage.transaction(true).then((tx) => {
-            let promises = [];
+        // noinspection BadExpressionStatementJS
+        const Selector = ItemKeyIndexChunkTupleSelector;
 
-            for (let encodedItemKeyIndexChunkTuple of encodedItemKeyIndexChunkTuples) {
-                promises.push(
-                    tx.saveTuplesEncoded(
-                        new ItemKeyIndexChunkTupleSelector(
-                            encodedItemKeyIndexChunkTuple.chunkKey
-                        ),
-                        encodedItemKeyIndexChunkTuple.encodedData
-                    )
-                );
-            }
+        if (tuplesToSave.length == 0) return;
 
-            return Promise.all(promises).then(() => tx.close());
-        });
-        return retPromise;
+        const batchStore: TupleStorageBatchSaveArguments[] = [];
+        for (const tuple of tuplesToSave) {
+            batchStore.push({
+                tupleSelector: new Selector(tuple.chunkKey),
+                vortexMsg: tuple.encodedData,
+            });
+        }
+
+        await this.storage.batchSaveTuplesEncoded(batchStore);
+
+        for (const tuple of tuplesToSave) {
+            this.index.updateDateByChunkKey[tuple.chunkKey] = tuple.lastUpdate;
+        }
+        await this.saveChunkCacheIndex(true);
+    }
+
+    /** Store Chunk Cache Index
+     *
+     * Updates our running tab of the update dates of the cached chunks
+     *
+     */
+    private async saveChunkCacheIndex(force = false): Promise<void> {
+        if (
+            this.chunksSavedSinceLastIndexSave <= this.SAVE_POINT_ITERATIONS &&
+            !force
+        ) {
+            return;
+        }
+
+        this.chunksSavedSinceLastIndexSave = 0;
+
+        await this.storage.saveTuples(new UpdateDateTupleSelector(), [
+            this.index,
+        ]);
     }
 
     /** Get Segment Keys
@@ -466,10 +482,8 @@ export class ItemKeyIndexLoaderService extends NgLifeCycleEvents {
         }
 
         // If there is no offline support, or we're online
-        if (
-            !this.deviceCacheControllerService.cachingEnabled ||
-            this.vortexStatusService.snapshot.isOnline
-        ) {
+        // This is never used at present.
+        if (this.vortexStatusService.snapshot.isOnline) {
             let ts = new TupleSelector(GraphDbPackedItemKeyTuple.tupleName, {
                 modelSetKey: modelSetKey,
                 keys: keys,
@@ -503,6 +517,15 @@ export class ItemKeyIndexLoaderService extends NgLifeCycleEvents {
                 });
         }
 
+        if (!this.deviceCacheControllerService.offlineModeEnabled) {
+            throw new Error(
+                "Peek is not online," +
+                    " and offline caching is not enabled" +
+                    " or has not completed loading." +
+                    " The ItemKeyIndex won't work."
+            );
+        }
+
         // If we do have offline support
         if (this.isReady()) return this.getItemKeysWhenReady(modelSetKey, keys);
 
@@ -557,7 +580,7 @@ export class ItemKeyIndexLoaderService extends NgLifeCycleEvents {
      * Get the objects with matching keywords from the index..
      *
      */
-    private getItemKeysForKeys(
+    private async getItemKeysForKeys(
         keys: string[],
         modelSetKey: string,
         chunkKey: string
@@ -567,43 +590,35 @@ export class ItemKeyIndexLoaderService extends NgLifeCycleEvents {
             return Promise.resolve([]);
         }
 
-        let retPromise: any;
-        retPromise = this.storage
-            .loadTuplesEncoded(new ItemKeyIndexChunkTupleSelector(chunkKey))
-            .then((vortexMsg: string) => {
-                if (vortexMsg == null) {
-                    return [];
-                }
+        const vortexMsg: string = await this.storage.loadTuplesEncoded(
+            new ItemKeyIndexChunkTupleSelector(chunkKey)
+        );
+        if (vortexMsg == null) {
+            return [];
+        }
 
-                return Payload.fromEncodedPayload(vortexMsg)
-                    .then((payload: Payload) => JSON.parse(<any>payload.tuples))
-                    .then((chunkData: { [key: number]: string }) => {
-                        let foundItemKeyIndexs: ItemKeyTuple[] = [];
+        const payload: Payload = await Payload.fromEncodedPayload(vortexMsg);
+        const chunkData: { [key: number]: string } = JSON.parse(
+            <any>payload.tuples[0]
+        );
+        let foundItemKeyIndexs: ItemKeyTuple[] = [];
 
-                        for (let key of keys) {
-                            // Find the keyword, we're just iterating
-                            if (!chunkData.hasOwnProperty(key)) {
-                                console.log(
-                                    `WARNING: ItemKeyIndex ${key} is missing from index,` +
-                                        ` chunkKey ${chunkKey}`
-                                );
-                                continue;
-                            }
+        for (let key of keys) {
+            // Find the keyword, we're just iterating
+            if (!chunkData.hasOwnProperty(key)) {
+                console.log(
+                    `WARNING: ItemKeyIndex ${key} is missing from index,` +
+                        ` chunkKey ${chunkKey}`
+                );
+                continue;
+            }
 
-                            let packedJson = chunkData[key];
-                            foundItemKeyIndexs.push(
-                                new ItemKeyTuple().unpackJson(
-                                    key,
-                                    packedJson,
-                                    modelSetKey
-                                )
-                            );
-                        }
+            let packedJson = chunkData[key];
+            foundItemKeyIndexs.push(
+                new ItemKeyTuple().unpackJson(key, packedJson, modelSetKey)
+            );
+        }
 
-                        return foundItemKeyIndexs;
-                    });
-            });
-
-        return retPromise;
+        return foundItemKeyIndexs;
     }
 }
